----------------------------------------------------------------------------
--Table for storing history of blocked spids.
----------------------------------------------------------------------------
SET ANSI_PADDING ON 
GO 
CREATE TABLE [dbo].[DBA_tblSP_WHO2] 
( 
    [SPID] [int] NULL , 
    [Status] [varchar] ( 500 ) COLLATE SQL_Latin1_General_CP1_CI_AS NULL , 
    [Login] [varchar] ( 500 ) COLLATE SQL_Latin1_General_CP1_CI_AS NULL , 
    [HOSTNAME] [varchar] ( 500 ) COLLATE SQL_Latin1_General_CP1_CI_AS NULL , 
    [BLKBY] [varchar] ( 500 ) COLLATE SQL_Latin1_General_CP1_CI_AS NULL , 
    [DBName] [varchar] ( 500 ) COLLATE SQL_Latin1_General_CP1_CI_AS NULL , 
    [Command] [varchar] ( 500 ) COLLATE SQL_Latin1_General_CP1_CI_AS NULL , 
    [CPUTime] [bigint] NULL , 
    [DiskIO] [bigint] NULL , 
    [LastBatch] [varchar] ( 500 ) COLLATE SQL_Latin1_General_CP1_CI_AS NULL , 
    [ProgramName] [varchar] ( 500 ) COLLATE SQL_Latin1_General_CP1_CI_AS NULL , 
    [SPID2] [bigint] NULL , 
    [REQUESTID] [int] NULL , 
    [dtCreated] [datetime] NOT NULL 
) 
ON [PRIMARY] 

GO 
---------------------------------------------------------------------------
-- Main proc for alerts 
---------------------------------------------------------------------------
GO 
/*************************************************************************************
AUTHOR:  JOSHUA A WALKER
DATE:  20110224
PURPOSE  CAPTURE SP_WHO2 DATA TO TABLE AND ALERT ABOUT BLOCKING

MODIFICATIONS:
JAW - 20110224 - INITIAL CREATION

TESTING:
EXEC DBO.DBA_spWho2CaptureAlert 
@db = 'Master'
, @TO = 'SOMEONE@SOMEWHERE.COM'

SP_WHO2
select * from DBA_tblSP_WHO2 
*************************************************************************************/ 
CREATE PROCEDURE [dbo].[DBA_spWho2CaptureAlert] 
@DB 
AS varchar ( 500 ) = NULL 
, @TO varchar ( 500 ) = NULL 
AS 

SET NOCOUNT ON; 

BEGIN 
    TRY 
    DROP TABLE #WHO2 
END 
TRY 
BEGIN 
    CATCH 
    --do nothing
END 
CATCH 

CREATE TABLE #WHO2 
( 
    SPID int 
    , Status varchar ( 500 ) , [Login] varchar ( 500 ) , HOSTNAME varchar ( 500 ) , BLKBY varchar ( 500 ) , DBName varchar ( 500 ) , Command varchar ( 500 ) , CPUTime bigint 
    , DiskIO bigint 
    , LastBatch varchar ( 500 ) , ProgramName varchar ( 500 ) , SPID2 bigint 
    , REQUESTID int 
) 

DECLARE @SQL varchar ( max ) 
SELECT @SQL = 'SP_WHO2' 

INSERT INTO #WHO2 

EXEC ( @SQL ) 

DELETE FROM DBA_tblSP_WHO2 
WHERE dtcreated <= dateadd ( day , - 14 , getdate ( ) ) 

INSERT INTO DBA_tblSP_WHO2 --SELECT * FROM DBA_tblSP_WHO2
( 
    SPID 
    , Status 
    , [Login] 
    , HOSTNAME 
    , BLKBY 
    , DBName 
    , Command 
    , CPUTime 
    , DiskIO 
    , LastBatch 
    , ProgramName 
    , SPID2 
    , REQUESTID 
    , dtCreated 
) 
SELECT 
SPID , 
Status , 
[Login] , 
HOSTNAME , 
BLKBY , 
DBName , 
Command , 
CPUTime , 
DiskIO , 
LastBatch , 
ProgramName , 
SPID2 , 
REQUESTID , 
getdate ( ) [dtCreated] 
FROM 
#WHO2 
WHERE SPID IN 
( 
    SELECT isnull ( replace ( replace ( cast ( BLKBY AS varchar ) , ' ' , '' ) , '.' , '' ) , '' ) [SPID] FROM #WHO2 
) 
OR isnull ( replace ( replace ( cast ( BLKBY AS varchar ) , ' ' , '' ) , '.' , '' ) , '' ) <> '' 
ORDER BY BLKBY , 
SPID 

DECLARE @RECCOUNT int 

IF isnull ( @DB , '' ) = '' 
BEGIN 
    SELECT @RECCOUNT = 
    ( 
        SELECT count ( * ) FROM #WHO2 
        WHERE 
        ( 
            SPID IN 
            ( 
                SELECT isnull ( replace ( replace ( cast ( BLKBY AS varchar ) , ' ' , '' ) , '.' , '' ) , '' ) [SPID] FROM #WHO2 
            ) 
            OR isnull ( replace ( replace ( cast ( BLKBY AS varchar ) , ' ' , '' ) , '.' , '' ) , '' ) <> '' 
        ) 
        AND 
        ( cast ( SPID AS varchar ) <> cast ( BLKBY AS varchar ) 
            AND cast ( BLKBY AS varchar ) NOT LIKE '%.%' ) 
    ) 

END 
ELSE 
BEGIN 
    SELECT @RECCOUNT = 
    ( 
        SELECT count ( * ) FROM #WHO2 
        WHERE 
        ( 
            SPID IN 
            ( 
                SELECT isnull ( replace ( replace ( cast ( BLKBY AS varchar ) , ' ' , '' ) , '.' , '' ) , '' ) [SPID] FROM #WHO2 
            ) 
            OR isnull ( replace ( replace ( cast ( BLKBY AS varchar ) , ' ' , '' ) , '.' , '' ) , '' ) <> '' 
        ) 
        AND 
        ( cast ( SPID AS varchar ) <> cast ( BLKBY AS varchar ) 
            AND cast ( BLKBY AS varchar ) NOT LIKE '%.%' 
            AND DBNAME = @DB ) 
    ) 

END 

IF @RECCOUNT > 0 
BEGIN 

    DECLARE @BODY varchar ( max ) , @POS int 
    , @CNTR int 
    , @SPID varchar ( 500 ) , @BLKBY varchar ( 500 ) , @EventInfo varchar ( max ) , @LASTSPID varchar ( 500 ) , @LASTBLKBY varchar ( 500 ) , @LOGIN varchar ( 500 ) , @DBNAME varchar ( 500 ) 

    DECLARE @WHO2MAIL TABLE 
    ( 
        [ID] int IDENTITY ( 1 , 1 ) , SPID int 
        , BLKBY varchar ( 500 ) , [LOGIN] varchar ( 500 ) , [DBNAME] varchar ( 500 ) 
    ) 

    INSERT INTO @WHO2MAIL ( SPID , BLKBY , [LOGIN] , [DBNAME] ) 
    SELECT 
    SPID , 
    BLKBY , 
    [LOGIN] , 
    [DBNAME] 
    FROM 
    #WHO2 
    WHERE SPID IN 
    ( 
        SELECT isnull ( replace ( replace ( cast ( BLKBY AS varchar ) , ' ' , '' ) , '.' , '' ) , '' ) [SPID] FROM #WHO2 
    ) 
    OR isnull ( replace ( replace ( cast ( BLKBY AS varchar ) , ' ' , '' ) , '.' , '' ) , '' ) <> '' 
    ORDER BY BLKBY , 
    SPID 

    BEGIN 
        TRY 
        DROP TABLE #INPUTBUFFER 
    END 
    TRY 
    BEGIN 
        CATCH 
    END 
    CATCH 

    CREATE TABLE #INPUTBUFFER 
    ( 
        EventType varchar ( max ) , Parameters varchar ( max ) , EventInfo varchar ( max ) 
    ) 

    SELECT 
    @POS = 1 , 
    @CNTR = max ( [ID] ) , 
    @BODY = '' , 
    @LASTSPID = '' , 
    @LASTBLKBY = '' , 
    @LOGIN = '' , 
    @DBNAME = '' 
    FROM @WHO2MAIL 

    SELECT @TO = isnull ( @TO , 'DEFAUTEMAILADDRESSES@SOMEWHERE.COM;' ) -- <--Put your default email address here.

    WHILE @POS <= @CNTR 
    BEGIN 
        SELECT 
        @SPID = cast ( SPID AS varchar ) , 
        @BLKBY = cast ( BLKBY AS varchar ) , 
        @LOGIN = cast ( isnull ( [LOGIN] , '' ) AS varchar ) , 
        @DBNAME = cast ( isnull ( [DBNAME] , '' ) AS varchar ) 
        FROM @WHO2MAIL WHERE ID = @POS TRUNCATE TABLE #INPUTBUFFER 

        --IF @POS = 1 
        --   BEGIN
        ----Code to capture first person and include in the email can be added here. 
        --END

        SELECT @SQL = 'DBCC INPUTBUFFER(' + @SPID + ')' 

        PRINT @SQL 

        INSERT INTO #INPUTBUFFER ( EventType , Parameters , EventInfo ) 

        EXEC ( @SQL ) 

        SELECT @EventInfo = isnull ( EventInfo , '' ) FROM #INPUTBUFFER 

        IF @LASTBLKBY = @BLKBY AND @LASTSPID = @SPID 
        BEGIN 
            SELECT @BODY = @BODY 

        END 
        ELSE 
        BEGIN 
            SELECT 

            @BODY = @BODY + '
            ' + @SPID + ' IS BEING BLOCKED BY ' + @BLKBY + ' Login: ' + @LOGIN + ' DBName: ' + @DBNAME + '
            INPUTBUFFER: ' + @EventInfo + '
            ' 
            , 
            @LASTBLKBY = @BLKBY , 
            @LASTSPID = @SPID 

        END 
        SELECT @POS = @POS + 1 

    END 

    EXEC DBO.DBA_spSendMail -- <--Whatever proc you use to send email... This proc is included below.
    @TO = @TO 
    , @SUBJECT = 'DBA_spWho2CaptureAlert - BLOCKED SPIDS ALERT!!!' 
    , @BODY = @Body 
END 

BEGIN 
    TRY 
    DROP TABLE #WHO2 
END 
TRY 
BEGIN 
    CATCH 
END 
CATCH 

BEGIN 
    TRY 
    DROP TABLE #INPUTBUFFER 
END 
TRY 
BEGIN 
    CATCH 
END 
CATCH 

GO 
----------------------------------------------------------------------------
--Proc for sending email using DBMail... You can use yours if you have one instead.
----------------------------------------------------------------------------

/**************************************************************************************
NAME:  JOSHUA A WALKER
DATE:  20080820
PURPOSE: SEND EMAIL. 

MODIFICATIONS:
JAW - 20080820 - INITIAL CREATION

NOTES:
[ @file_attachments = ] 'file_attachments'
Is a semicolon-delimited list of file names to attach to the e-mail message. Files in the list must be specified as absolute paths. 

TESTING:
EXEC DBO.DBA_spSendMail
@TO = 'SOMEONE@SOMEWHERE.COM'
, @SUBJECT = 'TESTING'
, @BODY = 'This is a test'
--, @CC = 'SOMEONE@SOMEWHERE.COM'
--, @BCC = 'SOMEONE@SOMEWHERE.COM'
, @file_attachments = 'C:\SOMEFILE.log'
**************************************************************************************/ 
ALTER PROCEDURE [dbo].[DBA_spSendMail] 
@TO varchar ( 8000 ) , @SUBJECT varchar ( 255 ) , @BODY nvarchar ( max ) , @CC varchar ( 8000 ) = NULL 
, @BCC varchar ( 8000 ) = NULL 
, @profile_name varchar ( 255 ) = 'Whatever your mail profile is called' 
, @file_attachments nvarchar ( max ) = NULL 

AS 

SET NOCOUNT ON 

-----------------------------------------------------------------------------
--SELECT @BODY = @BODY + '

*** SYSTEM ACCOUNT: PLEASE DO NOT REPLY TO THIS MESSAGE *** '
SELECT @BODY = @BODY + ' 

*** SYSTEM ACCOUNT ( ' + CAST(@@SERVERNAME AS VARCHAR) + ' ) : PLEASE DO NOT REPLY TO THIS MESSAGE *** '
-----------------------------------------------------------------------------

BEGIN TRY
EXEC msdb.dbo.sp_send_dbmail @recipients = @TO
, @subject = @SUBJECT
, @body = @BODY
, @copy_recipients = @CC
, @blind_copy_recipients = @BCC
, @body_format = ' HTML '--Allows for 
tags for new lines.
, @profile_name = @profile_name
, @file_attachments = @file_attachments
END TRY
BEGIN CATCH
--Your error capture code here... commented mine out... 

--INSERT INTO DBAdmin.dbo.DBA_tblErrorLog(Location, Script, ErrNumber, ErrDesc)
--SELECT ' DBAdmin.dbo.DBA_spSendMail ' [Location]
--, RIGHT(' 

EXEC msdb.dbo.sp_send_dbmail TO: ' + @TO + ' Subject: ' + @subject + ' Body: ' + @body,2000) [Script]
--, ERROR_NUMBER() [ErrNumber]
--, ERROR_MESSAGE() [ErrDesc]
END CATCH

GO
--------------------------------------------------------------------
-- Job for running this proc every 2 minutes.
--------------------------------------------------------------------
GO

USE [msdb]
GO
/****** Object: Job [[MAINT]] Blocked Process ALERT] Script Date: 03/25/2011 12:07:46 ******/
BEGIN TRANSACTION
DECLARE @ReturnCode INT
SELECT @ReturnCode = 0
/****** Object: JobCategory [[Uncategorized (Local)]]] Script Date: 03/25/2011 12:07:46 ******/
IF NOT EXISTS (SELECT name FROM msdb.dbo.syscategories WHERE name=N' [Uncategorized (Local)] ' AND category_class=1)
BEGIN
EXEC @ReturnCode = msdb.dbo.sp_add_category @class=N' JOB ', @type=N' LOCAL ', @name=N' [Uncategorized (Local)] '
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback

END

DECLARE @jobId BINARY(16)
EXEC @ReturnCode = msdb.dbo.sp_add_job @job_name=N' [MAINT] Blocked Process ALERT ', 
@enabled=0, --ENABLE THE JOB ONCE YOU GOT EVERYTHING IN PLACE... DEFAULT IS DISABLED.
@notify_level_eventlog=0, 
@notify_level_email=2, 
@notify_level_netsend=0, 
@notify_level_page=0, 
@delete_level=0, 
@description=N' NO description available. ', 
@category_name=N' [Uncategorized (Local)] ', 
@owner_login_name=N' sa ', 
@notify_email_operator_name=N' OperatorName ', @job_id = @jobId OUTPUT -- <--Your operator name here... 
IF (@@ERROR <> 0 OR @ReturnCode <> 0) GOTO QuitWithRollback
/****** Object: Step [DUMMY START] Script Date: 03/25/2011 12:07:46 ******/
EXEC @ReturnCode = msdb.dbo.sp_add_jobstep @job_id=@jobId, @step_name=N' DUMMY START ', 
@step_id=1, 
@cmdexec_success_code=0, 
@on_success_action=3, 
@on_success_step_id=0, 
@on_fail_action=3, 
@on_fail_step_id=0, 
@retry_attempts=0, 
@retry_interval=0, 
@os_run_priority=0, @subsystem=N' TSQL ', 
@command=N' --DO NOTHING', 
@database_name = N'master' , 
@flags = 0 
IF ( @@ERROR <> 0 OR @ReturnCode <> 0 ) 
GOTO QuitWithRollback 
/****** Object: Step [EXEC PROC] Script Date: 03/25/2011 12:07:46 ******/ 

EXEC @ReturnCode = msdb.dbo.sp_add_jobstep @job_id = @jobId , @step_name = N'EXEC PROC' , 
@step_id = 2 , 
@cmdexec_success_code = 0 , 
@on_success_action = 3 , 
@on_success_step_id = 0 , 
@on_fail_action = 2 , 
@on_fail_step_id = 0 , 
@retry_attempts = 1 , 
@retry_interval = 1 , 
@os_run_priority = 0 , @subsystem = N'TSQL' , 
@command = N'EXEC DBO.DBA_spWho2CaptureAlert' , 
@database_name = N'master' , --Your DB Name where this runs here... 
@flags = 0 
IF ( @@ERROR <> 0 OR @ReturnCode <> 0 ) 
GOTO QuitWithRollback 
/****** Object: Step [DUMMY CLOSE] Script Date: 03/25/2011 12:07:46 ******/ 

EXEC @ReturnCode = msdb.dbo.sp_add_jobstep @job_id = @jobId , @step_name = N'DUMMY CLOSE' , 
@step_id = 3 , 
@cmdexec_success_code = 0 , 
@on_success_action = 1 , 
@on_success_step_id = 0 , 
@on_fail_action = 1 , 
@on_fail_step_id = 0 , 
@retry_attempts = 0 , 
@retry_interval = 0 , 
@os_run_priority = 0 , @subsystem = N'TSQL' , 
@command = N'--DO NADA' , 
@database_name = N'master' , 
@flags = 0 
IF ( @@ERROR <> 0 OR @ReturnCode <> 0 ) 
GOTO QuitWithRollback 

EXEC @ReturnCode = msdb.dbo.sp_update_job @job_id = @jobId , @start_step_id = 1 
IF ( @@ERROR <> 0 OR @ReturnCode <> 0 ) 
GOTO QuitWithRollback 

EXEC @ReturnCode = msdb.dbo.sp_add_jobschedule @job_id = @jobId , @name = N'EVERY 2 MIN FROM 7 TO 7 - 1' , 
@enabled = 1 , 
@freq_type = 8 , 
@freq_interval = 62 , 
@freq_subday_type = 4 , 
@freq_subday_interval = 2 , 
@freq_relative_interval = 0 , 
@freq_recurrence_factor = 1 , 
@active_start_date = 20110224 , 
@active_end_date = 99991231 , 
@active_start_time = 70000 , 
@active_end_time = 190000 
IF ( @@ERROR <> 0 OR @ReturnCode <> 0 ) 
GOTO QuitWithRollback 

EXEC @ReturnCode = msdb.dbo.sp_add_jobserver @job_id = @jobId , @server_name = N'(local)' 
IF ( @@ERROR <> 0 OR @ReturnCode <> 0 ) 
GOTO QuitWithRollback 
COMMIT TRANSACTION 
GOTO EndSave 
QuitWithRollback: 
IF ( @@TRANCOUNT > 0 ) ROLLBACK TRANSACTION 
EndSave: 
---------------------------------------------------------------------------